unit Cube;

interface

type
  PMoves = ^TMoves;
  TMoves = array[1..200] of ShortInt;

  TSequ = record
    Moves : Byte;
    Move: array[1..20]of ShortInt;
  end;

  TSequs = array[1..100]of TSequ;

  TPosition = record
    Side,Index : ShortInt; //Just in case it is signed
  end;

  PFace = ^TFace;
  TFace = record
    Color : Byte;
    Cubie : Byte;
  end;
  TFaceArray = array[1..8]of TFace;

  PSide = ^TSide;
  TSide = record
    SideFace: array[1..12]of PFace;
    FrontFace: TFaceArray;
  end;

  TRCube=class
  private
    FSolving: Boolean;
    FMoves : PMoves;
    FIndex : Integer;
    FCubeRot: Byte;
    FLoaded : Boolean;
    FSequs : TSequs;
    FSide: array[1..6]of PSide;
    function LoadMvs: Boolean;
    procedure FixPos(var Pos: TPosition);
    function FixMove(AMove: Integer): Integer;
    procedure SwitchCorners(Color: Integer);
    procedure RotateCorners(Dir: Boolean);
    procedure Set3Corners;
    procedure DoCrossCorners;
    procedure Do2Corners(Rot: Integer);
    procedure RotEdges(Right:Boolean);
    procedure Step1;  //Solves 2 layers
    procedure Step2;  //Solves 3. layer's corners
    procedure Step3;  //Solves 3. layer's edges
  public
    Side: array[1..6]of TSide;
    function Find(AColor,ACubie: Byte): TPosition;
    procedure SetCubie(AColor,ACubie: Byte;Pos: TPosition);
    procedure ResetFaces;
    procedure ResetSides;
    procedure Rotate(ASide: Integer);
    procedure RotateCube(Times: Integer);
    function Solve(var AMoves: TMoves): Integer;
    constructor Create;
  end;

function OptimizeMoves(var AMoves: TMoves;No: Integer): Integer;

const
  csWhite = 1;
  csGreen = 2;
  csRed = 3;
  csBlue = 4;
  csOrange = 5;
  csYellow = 6;
  csTop = csWhite;
  csBottom = csYellow;
  csFront = csBlue;
  csBack = csGreen;
  csLeft = csOrange;
  csRight = csRed;

  SideIndex : array[1..6,0..3]of Byte=
      ((2,3,4,5), (1,5,6,3),
       (1,2,6,4), (1,3,6,5),
       (1,4,6,2), (4,3,2,5));

implementation

uses Classes, SysUtils;

const Cubies: array[1..6,1..8]of Byte =
      (( 1, 2, 3, 4, 5, 6, 7, 8),
       ( 3, 2, 1, 9,13,14,15,10),
       ( 5, 4, 3,10,15,16,17,11),
       ( 7, 6, 5,11,17,18,19,12),
       ( 1, 8, 7,12,19,20,13, 9),
       (19,18,17,16,15,14,13,20));

{ TRCube }

constructor TRCube.Create;
const
  NA: array[1..6,1..12]of Byte=
      ((3,2,1,3,2,1,3,2,1,3,2,1),
       (3,2,1,1,8,7,7,6,5,5,4,3),
       (5,4,3,1,8,7,5,4,3,5,4,3),
       (7,6,5,1,8,7,3,2,1,5,4,3),
       (1,8,7,1,8,7,1,8,7,5,4,3),
       (7,6,5,7,6,5,7,6,5,7,6,5));
var i,n: Integer;
begin
  inherited;

  ResetFaces;

  for i := 1 to 6 do
    for n := 1 to 12 do
      Side[i].SideFace[n] :=
        @Side[SideIndex[i,(n-1)div 3]].FrontFace[NA[i,n]];

  ResetSides;
end;

procedure TRCube.Do2Corners(Rot: Integer);
begin
  if rot<3 then Inc(rot,2)
  else Dec(rot,2);
  if Side[6].SideFace[rot*3]^.Color=6 then
    rot := (5-rot)and 3
  else
    Rot := 4-rot;
  RotateCube(rot);
  RotateCorners(True);
  ResetSides;
end;

procedure TRCube.DoCrossCorners;
begin
  with Side[6] do
    if FrontFace[1].Color = 6 then
    begin
      RotateCube(1);
      RotateCorners(SideFace[3]^.Color=6);
      ResetSides;
    end
    else
      RotateCorners(SideFace[6]^.Color=6);
end;

function TRCube.Find(AColor, ACubie: Byte): TPosition;
var s,i: Integer;
begin
  FillChar(Result,SizeOf(TPosition),0);
  for s := 1 to 6 do
    for i := 1 to 8 do
      with FSide[s]^.FrontFace[i] do
        if (Color=AColor)and(Cubie=ACubie)then
        begin
          Result.Side := s;
          Result.Index := i;
          Exit;
        end;
end;

function TRCube.FixMove(AMove: Integer): Integer;
var S: ShortInt;
begin
  Result := AMove;
  S := Abs(AMove) div AMove;          //-1 or 1
  AMove := AMove*S;                   //Abs(Move)
  if (AMove mod 6<2)or(FCubeRot=0) then Exit;

  Result := ((AMove+FCubeRot-2) and 3 + 2)*S;
end;

procedure TRCube.FixPos(var Pos: TPosition);
begin
  if (Pos.Side<>1)and(Pos.Side<>6)or(FCubeRot=0)then Exit;

  if FCubeRot=2 then
  begin
    if Pos.Index<5 then
      Inc(Pos.Index,4)
    else
      Dec(Pos.Index,4);
    Exit;
  end;

  Dec(Pos.Index);
  with Pos do
    if (Side-FCubeRot)mod 3=0 then  //11 or 63
      Index := (Index+6)and 7
    else
      Index := (Index+2)and 7;
  Inc(Pos.Index);
end;

function TRCube.LoadMvs: Boolean;
const Msg = 'Sorry but you have to pay me first!';
var Dat: TFileStream;
    ID : array[0..2]of Char;
    i : Integer;
    SeqsNum : Byte;
begin
  Result := FLoaded;
  if Result then Exit;

  Result := False;

  if not FileExists('Moves.dat') then
    raise Exception.Create(Msg);
  Dat := TFileStream.Create('Moves.dat',fmOpenRead);
  try
    Dat.Read(ID,SizeOf(ID));
    if ID<>'MVS' then
      raise Exception.Create(Msg);
    Dat.Read(SeqsNum,SizeOf(SeqsNum));
    for i := 1 to SeqsNum do
    begin
      Dat.Read(FSequs[i].Moves,1);
      Dat.Read(FSequs[i].Move,FSequs[i].Moves);
    end;
    FLoaded := True;
    Result := True;
   finally
    Dat.Free;
  end;
end;

procedure TRCube.ResetFaces;
var i,n: Integer;
begin
  //Color the cube
  for i := 1 to 6 do
    for n := 1 to 8 do
    begin
      Side[i].FrontFace[n].Color := i;
      Side[i].FrontFace[n].Cubie := Cubies[i,n];
    end;
end;

procedure TRCube.ResetSides;
var i : Integer;
begin
  //Connect pointers with sides
  for i := 1 to 6 do
    FSide[i] := @Side[i];
  FCubeRot := 0;
end;

procedure TRCube.Rotate(ASide: Integer);
var s,i: Byte;
    Tmp: array[1..3]of TFace;
begin
  if FSolving then
  begin
    Inc(FIndex);
    FMoves^[FIndex] := FixMove(ASide);
  end;

  s := Abs(ASide);
  with FSide[s]^ do
    if ASide>0 then
    begin
      Tmp[1] := FrontFace[7];
      Tmp[2] := FrontFace[8];
      for i := 8 downto 3 do
        FrontFace[i] := FrontFace[i-2];
      FrontFace[1] := Tmp[1];
      FrontFace[2] := Tmp[2];

      for i := 12 downto 4 do
      begin
        if i>9 then Tmp[i-9] := SideFace[i]^;
        SideFace[i]^ := SideFace[i-3]^;
        if i<7 then SideFace[i-3]^ := Tmp[i-3];
      end;
    end
    else
    begin
      Tmp[1] := FrontFace[1];
      Tmp[2] := FrontFace[2];
      for i := 1 to 6 do
        FrontFace[i] := FrontFace[i+2];
      FrontFace[7] := Tmp[1];
      FrontFace[8] := Tmp[2];

      for i := 1 to 9 do
      begin
        if i<4 then Tmp[i] := SideFace[i]^;
        SideFace[i]^ := SideFace[i+3]^;
        if i>6 then SideFace[i+3]^ := Tmp[i-6];
      end;
    end;
end;

procedure TRCube.RotateCorners(Dir: Boolean);
var i: Integer;
begin
  if Dir then
    for i := 1 to FSequs[63].Moves do
      Rotate(FSequs[63].Move[i])
  else
    for i := FSequs[63].Moves downto 1 do
      Rotate(-FSequs[63].Move[i]);
end;

procedure TRCube.RotateCube(Times: Integer);
var i,n: Integer;
    T : PSide;
begin
  for i := 1 to Times do
  begin
    T := FSide[2];
    for n := 2 to 4 do
      FSide[n] := FSide[n+1];
    FSide[5] := T;
  end;
  FCubeRot := (FCubeRot+Times)and 3;
end;

procedure TRCube.RotEdges(Right: Boolean);
var i,seq : Integer;
begin
  seq := Byte(Right)+64;
  for i := 1 to FSequs[seq].Moves do
    Rotate(FSequs[seq].Move[i]);
end;

procedure TRCube.Set3Corners;
var rot : Byte;
    dir : Boolean;
begin
  with Side[6] do
  begin
    if FrontFace[7].Color=6 then rot := 1
    else if FrontFace[3].Color=6 then rot := 3
    else if FrontFace[5].Color=6 then rot := 2
    else rot := 0;
    if rot>0 then
    begin
      RotateCube(rot);
      dir := SideFace[12]^.Color=6;
    end
    else
      dir := SideFace[3]^.Color=6;
    RotateCorners(dir);
    if rot>0 then ResetSides;
  end;
end;

procedure TRCube.SetCubie(AColor, ACubie: Byte;Pos: TPosition);
  procedure SwitchFaces(var A,B: TFace);
  var Temp : TFace;
  begin
    Temp := A;
    A := B;
    B := Temp;
  end;

var SPos: TPosition;
    Corner: Boolean;
begin
  SPos := Find(AColor,ACubie);
  if (SPos.Side=Pos.Side)and(SPos.Index=Pos.Index) then Exit;
  Corner := Odd(Pos.Index);
  SwitchFaces(Side[Pos.Side].FrontFace[Pos.Index],
              Side[SPos.Side].FrontFace[SPos.Index]);
  with Pos do Inc(Index,Index div 2);
  if Side[SPos.Side].FrontFace[SPos.Index].Cubie=ACubie then
  begin  //Just Rotate Colors on the Cubie
    if Corner then
    begin
      if Side[Pos.Side].SideFace[Pos.Index]^.Color =
         Side[SPos.Side].FrontFace[SPos.Index].Color then
        with Pos do
        begin
          Dec(Index);
          if Index=0 then Index := 12;
        end;
      SwitchFaces(Side[SPos.Side].FrontFace[SPos.Index],
                 Side[Pos.Side].SideFace[Pos.Index]^);
    end;
    Exit;
  end;
  with SPos do Inc(Index,Index div 2);
  if Corner then
    SwitchFaces(Side[Pos.Side].SideFace[Pos.Index]^,
                Side[SPos.Side].SideFace[SPos.Index]^);
  with SPos do
  begin
    Dec(Index);
    if Index=0 then Index := 12;
  end;
  with Pos do
  begin
    Dec(Index);
    if Index=0 then Index := 12;
  end;
  SwitchFaces(Side[Pos.Side].SideFace[Pos.Index]^,
              Side[SPos.Side].SideFace[SPos.Index]^);
end;

function TRCube.Solve(var AMoves: TMoves): Integer;
var Save: array[1..6,1..8]of TFace;
    s,i : Byte;
begin
  Result := 0;
  if not LoadMvs then Exit;

  ResetSides;
  FSolving := True;
  FMoves := @AMoves;
  FillChar(FMoves^,SizeOf(FMoves^),0);
  FIndex := 0;

  for s := 1 to 6 do
    for i := 1 to 8 do
      Save[s,i] := Side[s].FrontFace[i];

  Step1;   //Resava sredine i coske na prva 2 nivoa
  Step2;   //Resava coske na donjem nivou
  Step3;   //Resava sredine na donjem nivou
  ResetSides;

  FSolving := False;
  FMoves := nil;
  for s := 1 to 6 do
    for i := 1 to 8 do
      Side[s].FrontFace[i] := Save[s,i];

  Result := FIndex;
end;

procedure TRCube.Step1;
var ind,step,i,n,clrt,cbe,CurSeq : Integer;
    CurSide : PSide;
    CurStep : Boolean;
    Position : TPosition;

  function CheckWhite: Boolean;
  begin
    Result := (step=1)and(i=1);
    if Result then
    begin
      Result := (CurSeq<4);
      if Result then
        if CurSeq=1 then Rotate(-1)
        else if CurSeq=3 then Rotate(1)
        else begin
          Rotate(1);
          Rotate(1);
        end;
    end;
  end;

begin
  CurSide := @Side[1];
  clrt := 1;
  for step := 1 to 3 do
    for i := 1 to 4 do
    begin
      cbe := i*2;
      if step>1 then
        if Step=3 then
        begin
          CurSide := FSide[2];
          clrt := i+1;
          cbe := i+8;
        end
        else Dec(cbe);
      if step<3 then ind := cbe
      else ind := 4;
      with CurSide^.FrontFace[ind] do
        CurStep := (Color=clrt)and(Cubie=cbe);
      if not CurStep then
      begin
        Position := Find(clrt,cbe);
        FixPos(Position);
        with Position do
          if step<3 then CurSeq := (Index+(Side-1)*8)div 2 - 1
          else CurSeq := (Index+(Side-2)*6)div 2+44+Side div 6;
        if step=2 then Inc(CurSeq,24);
        if not CheckWhite then
          for n := 1 to FSequs[CurSeq].Moves do
            Rotate(FSequs[CurSeq].Move[n]);
      end;
      RotateCube(1);
    end;
end;

procedure TRCube.Step2;
var cbe1,cbe2,dif,clr,clr1,rot,i : Byte;
    goodpos : Boolean;
begin
  rot := 0;
  //Which two colors are together
  //Find matching cubies
  cbe1 := Side[6].FrontFace[1].Cubie;
  cbe2 := Side[6].FrontFace[3].Cubie;
  dif := Abs(cbe2-cbe1);
  if dif = 4 then
  begin
    cbe2 := cbe1;
    cbe1 := Side[6].FrontFace[7].Cubie;
    dif := Abs(cbe2-cbe1);
    rot := 1;
  end;

  //Then turn into color
  if dif=2 then
    clr := (cbe1+cbe2-20)div 4
  else
    clr := 5;

  //Get those colors on proper side
  case clr of
  2: Inc(rot,2);
  3: Inc(rot);
  5: Inc(rot,3);
  end;
  rot := rot and 3;

  if rot<3 then
    for i := 1 to rot do
      Rotate(6)
  else Rotate(-6);

  //Check Pos and get opposite color
  goodpos := (cbe1-cbe2=2)or(cbe1-cbe2=-6);
  clr1 :=  clr and 3 + 2;
  //Switch their positions if needed
  if not goodpos then
    SwitchCorners(clr);

  //Check and switch opposite corners
  with Side[6] do
    goodpos := (FrontFace[1].Cubie=19)and
               (FrontFace[3].Cubie=17)and
               (FrontFace[5].Cubie=15);
  if not goodpos then
    SwitchCorners(clr1);

  //All corners are in their position now
  //But what about their orientation? How many are good?
  with Side[6] do
  begin
    rot := Byte(FrontFace[1].Color=6)+
           Byte(FrontFace[3].Color=6)+
           Byte(FrontFace[5].Color=6)+
           Byte(FrontFace[7].Color=6);
    if rot=2 then
    begin
      clr := 0;
      for i := 1 to 4 do
        if SideFace[i*3-2]^.Color<>6 then
          if SideFace[i*3-2]^.Color=SideFace[i*3].Color then
            clr := i;
      if clr = 0 then DoCrossCorners
      else Do2Corners(clr);
    end
    else if rot = 0 then
    begin
      if (SideFace[3]^.Color=6)=(SideFace[6]^.Color=6)then
        goodpos := SideFace[3]^.Color<>6
      else
        goodpos := SideFace[10]^.Color=6;
      RotateCorners(goodpos);
    end;
    if rot<>4 then Set3Corners;
  end;
end;

procedure TRCube.Step3;
var goodpos,rot : Byte;
    RightRot: Boolean;
begin
  rot := 0;
  with Side[6] do
  begin
    goodpos := Byte(FrontFace[2].Cubie=18)+
               Byte(FrontFace[4].Cubie=16)+
               Byte(FrontFace[6].Cubie=14)+
               Byte(FrontFace[8].Cubie=20);

    if goodpos=0 then
      RotEdges(True);

    if goodpos<2 then  //It is one
    begin
      if FrontFace[4].Cubie=16 then rot := 3
      else if FrontFace[6].Cubie=14 then rot := 2
      else if FrontFace[8].Cubie=20 then rot := 1;
      case rot of
      0: RightRot := FrontFace[8].Cubie=16;
      1: RightRot := FrontFace[6].Cubie=18;
      2: RightRot := FrontFace[4].Cubie=20;
      else RightRot := FrontFace[2].Cubie=14;
      end;
      RotateCube(rot);
      RotEdges(RightRot);
      ResetSides;
    end; //if

    //Now all are in place but what about orientation

    goodpos := Byte(FrontFace[2].Color=6)+
               Byte(FrontFace[4].Color=6)+
               Byte(FrontFace[6].Color=6)+
               Byte(FrontFace[8].Color=6);
    if goodpos=4 then Exit;

    if goodpos =0 then
    begin
      RotEdges(True);
      RotEdges(False);
    end;

    if FrontFace[8].Color=FrontFace[4].Color then  //2 opposite are bad
    begin
      RotateCube(1);
      RotEdges(True);
      RotEdges(False);
    end
    else if FrontFace[2].Color=FrontFace[6].Color then
    begin
      RotEdges(True);
      RotEdges(False);
    end
    else //two not opposite are bad
    begin
      rot := 0;
      if FrontFace[2].Color=FrontFace[4].Color then
        rot := 3
      else if FrontFace[4].Color=FrontFace[6].Color then
        rot := 2
      else if FrontFace[6].Color=FrontFace[8].Color then
        rot := 1;
      RotateCube(rot);
      RotEdges(False);
      RotEdges(True);
    end;
  end; {With}
end;

procedure TRCube.SwitchCorners(Color: Integer);
var i,rot : Integer;
begin
  rot := 0;
  case Color of
  2: rot := 2;
  3: rot := 3;
  5: rot := 1;
  end;
  RotateCube(Rot);
  for i := 1 to FSequs[62].Moves do
    Rotate(FSequs[62].Move[i]);
  if Color<>4 then ResetSides;
end;

function OptimizeMoves(var AMoves: TMoves;No: Integer): Integer;
var i : Integer;
    T : TMoves;

  procedure StepOne;
  begin
    Result := 1;
    T := AMoves;
    i := 1;
    while i < No do
      if T[i]=-T[i+1] then Inc(i,2)
      else
      begin
        AMoves[Result] := T[i];
        Inc(Result);
        Inc(i);
      end;
    AMoves[Result]:= T[No];
  end;
begin
  StepOne;

  //Step Two
  No := Result;
  Result := 1;
  i := 1;
  T := AMoves;
  while i < No do
  begin
    if (T[i]=T[i+1])and(T[i]=T[i+2])then
      if T[i]=T[i+3] then
        Inc(i,4)      //Four same
      else
      begin
        AMoves[Result] := -T[i];  //Three same
        Inc(Result);
        Inc(i,3);
      end
    else
    begin
      AMoves[Result] := T[i];
      Inc(Result);
      Inc(i);
    end;
  end;
  if i=No then
    AMoves[Result] := T[i];

  No := Result;
  StepOne;
end;

end.
