unit Cube3D;

interface

uses Cube, gl;

procedure SetActiveCube(ARCube: TRCube);
procedure SetRotationSpeed(Speed: Integer);
procedure SetViewRotSpeed(Speed: Integer);
procedure DrawCube;
procedure StartViewRotation(X,Y: Integer);
procedure StopViewRotation;
function RotateSide(ASide: Integer): Boolean;
procedure RotateView(X,Y: Integer);
procedure RotateViewFor(X,Y: Integer);
function GetClickedSide(X,Y: Integer): ShortInt;
function GetClickedPos(X,Y: Integer): TPosition;
procedure CreateList;

var Colors: array[1..6,0..2]of GLubyte =
      ((255,255,255),(0,255,0),(255,0,0),
       (0,128,255),(255,128,0),(255,255,0));

implementation

uses Windows, Classes, Dialogs, SysUtils, glu;

const CP: array[0..8,1..3]of Double=
      ((-0.485, 1.5,-0.485),(-1.5, 1.5,-1.5),(-0.485, 1.5,-1.5),
       ( 0.53, 1.5,-1.5),( 0.53, 1.5,-0.485),( 0.53, 1.5, 0.53),
       (-0.485, 1.5, 0.53),(-1.5, 1.5, 0.53),(-1.5, 1.5,-0.485));

var RCube: TRCube;
    Face: GLuint;
    ViewRotating: Boolean;
    xvrot,yvrot,xvoldrot,yvoldrot,vrotspeed : Single;
    xrot,yrot,zrot: Integer;
    RotSpeed : Single;
    ClickPos: TPoint;
    SideRotation: ShortInt;
    FaceRotate: array[1..6,0..8]of Boolean;
    Selecting: Boolean = False;

procedure SetActiveCube(ARCube: TRCube);
begin
  RCube := ARCube;
  RotSpeed := 5/8;
  vrotspeed := 1/3;
  xvrot := 40;
  yvrot := -40;
  xvoldrot := xvrot;
  yvoldrot := yvrot;
end;

procedure SetRotationSpeed(Speed: Integer);
begin
  Speed := Abs(Speed);
  if Speed>100 then Speed := 100;
  RotSpeed := Speed/8;
end;

procedure SetViewRotSpeed(Speed: Integer);
begin
  if Speed>20 then Speed := 20;
  vrotspeed := 1/(11-speed/2);
end;

procedure ViewInit;
begin
  glLoadIdentity;
  glTranslatef(0,0,-7);
  glRotatef(xvrot,1,0,0);
  glRotatef(yvrot,0,1,0);
end;

procedure MoveToFace(ASide,AIndex: Integer);
begin
  case ASide of
  2: begin
       glRotatef(-90,1,0,0);
       glRotatef(180,0,1,0);
     end;
  3: glRotatef(-120,-1,-1,1);
  4: glRotatef(90,1,0,0);
  5: glRotatef(120,1,-1,1);
  6: glRotatef(180,1,0,0);
  end;
  glTranslatef(CP[AIndex,1],CP[AIndex,2],CP[AIndex,3]);
end;

procedure DrawInnerEdge(ASide,AIndex: Integer);
begin
  glColor3f(0.3,0.3,0.3);
  glTranslatef(0,-0.97,0);
  glCallList(Face);
  if Abs(SideRotation)=ASide then
  begin
    glPopMatrix;
    glPushMatrix;
    MoveToFace(ASide,AIndex);
    glTranslatef(0,-0.97,0);
  end;
  glTranslatef(0,-0.045,0);
  glCallList(Face);
end;

procedure DrawCube;
var side,i: Integer;
begin
  ViewInit;

  for side := 1 to 6 do
    for i := 0 to 8 do
    begin
      glPushMatrix;
      if (SideRotation<>0)and FaceRotate[side,i] then
        glRotatef(RotSpeed*(xrot+yrot+zrot),Abs(xrot),Abs(zrot),Abs(yrot));
      MoveToFace(side,i);
      if i=0 then glColor3ubv(@Colors[side,0])
      else with RCube.Side[side].FrontFace[i] do
        glColor3ubv(@Colors[Color,0]);
      glLoadName(side shl 4+i);
      glCallList(Face);
      if not Selecting then
        DrawInnerEdge(side,i);
      glPopMatrix;
    end;
  if SideRotation<>0 then
  begin
    if xrot<>0 then xrot := xrot+xrot div Abs(xrot);
    if yrot<>0 then yrot := yrot+yrot div Abs(yrot);
    if zrot<>0 then zrot := zrot+zrot div Abs(zrot);
    if Abs(xrot+yrot+zrot)>=90/RotSpeed then
    begin
      RCube.Rotate(SideRotation);
      SideRotation:= 0;
    end;
  end;
end;

procedure StartViewRotation(X,Y: Integer);
begin
  if ViewRotating then Exit;
  ViewRotating := True;
  ClickPos := Point(X,Y);
end;

procedure StopViewRotation;
begin
  ViewRotating := False;
  xvoldrot := xvrot;
  yvoldrot := yvrot;
end;

procedure RotateView(X,Y: Integer);
begin
  if not ViewRotating then Exit;
  xvrot := xvoldrot-(ClickPos.y-y)*vrotspeed;
  yvrot := yvoldrot-(ClickPos.x-x)*vrotspeed;
end;

procedure RotateViewFor(X,Y: Integer);
begin
  xvrot := xvrot+Y/6;
  yvrot := yvrot+X/6;
  xvoldrot := xvrot;
  yvoldrot := yvrot;
end;

function RotateSide(ASide: Integer): Boolean;
var i: Integer;
    Position : TPosition;
begin
  Result := not((SideRotation<>0)or(ASide=0));
  if not Result then Exit;
  SideRotation := ASide;
  ASide := Abs(ASide);
  i := SizeOf(FaceRotate);
  FillChar(FaceRotate,i,0);
  FillChar(FaceRotate[ASide],i div 6,True);
  for i := 1 to 12 do
  begin
    with RCube.Side[ASide].SideFace[i]^ do
      Position := RCube.Find(Color,Cubie);
    with Position do
      FaceRotate[Side,Index] := True;
  end;
  xrot := 0; yrot := 0; zrot := 0;
  case ASide of
  1: zrot := -SideRotation;
  6: zrot := SideRotation;
  4: yrot := -SideRotation;
  2: yrot := SideRotation;
  5: xrot := SideRotation;
  else xrot := -SideRotation;
  end;
  xrot := xrot div ASide;
  yrot := yrot div ASide;
  zrot := zrot div ASide;
end;

function GetClickedSide(X,Y: Integer): ShortInt;
begin
  Result := GetClickedPos(X,Y).Side;
end;

function GetClickedPos(X,Y: Integer): TPosition;
var Buffer : array[0..511]of GLuint;
    hits : GLint;
    viewport: TViewPortArray;
    choose,depth: GLuint;
    i: Integer;
begin
  FillChar(Result,SizeOf(TPosition),0);
  if ViewRotating then Exit;

  glGetIntegerv(GL_VIEWPORT,@viewport[0]);
  glSelectBuffer(511,@Buffer[0]);
  glRenderMode(GL_SELECT);

  glInitNames;
  glPushName(0);

  glMatrixMode(GL_PROJECTION);
  glPushMatrix;
  glLoadIdentity;
  gluPickMatrix(X,viewport[3]-Y+viewport[1],1,1,viewport);
  gluPerspective(45.0,viewport[2]/viewport[3],1,100);
  glMatrixMode(GL_MODELVIEW);

  Selecting := True;
  glEnable(GL_CULL_FACE);
  DrawCube;
  glDisable(GL_CULL_FACE);
  Selecting := False;

  glMatrixMode(GL_PROJECTION);
  glPopMatrix;
  glMatrixMode(GL_MODELVIEW);
  hits := glRenderMode(GL_RENDER);

  if hits=0 then Exit;

  choose := buffer[3];
  depth := buffer[1];
  for i := 1 to hits-1 do
    if buffer[i*4+1]<depth then
    begin
      choose := buffer[i*4+3];
      depth := buffer[i*4+1];
    end;
  Result.Side := choose shr 4;
  Result.Index := choose and $F;
end;

procedure CreateList;
begin
  Face := glGenLists(1);
  glNewList(Face,GL_COMPILE);
    glBegin(GL_QUADS);
      glVertex3d(   0, 0,   0);
      glVertex3d(   0, 0,0.97);
      glVertex3d(0.97, 0,0.97);
      glVertex3d(0.97, 0,   0);
    glEnd;
  glEndList;
end;

end.
