unit Main;

interface

uses sdl, SDLListBox, SDLLabel, SDLButton, 
  SDLFileCtrls, sdlgui, SDLDraw, SDLForm, Cube;

type
  TfmRCube = class(TSDLForm)
    btLoad: TSDLButton;
    btMix: TSDLButton;
    btNextMove: TSDLButton;
    btOptions: TSDLButton;
    btPlay: TSDLButton;
    btPrevMove: TSDLButton;
    btSave: TSDLButton;
    btSetCube: TSDLButton;
    btSolve: TSDLButton;
    lblMoves: TSDLLabel;
    lbMoves: TSDLListBox;
    SDLButton1: TSDLButton;
    SDLButton2: TSDLButton;
    SDLPanel1: TSDLPanel;
    procedure btLoadClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure btMixClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure btNextMoveClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure btOptionsClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure btPlayClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure btPrevMoveClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure btSaveClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure btSetCubeClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure btSolveClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure fmRCubeDestroy(Sender: TSDLComponent);
    procedure fmRCubeMouseMove(Sender: TSDLComponent; Modifier: Cardinal; AX, AY: Integer);
    procedure fmRCubeMouseUp(Sender: TSDLComponent; Button: Integer; Modifier: Cardinal; AX, AY: Word);
    procedure lbMovesClick(Sender: TSDLComponent; AX, AY: Integer);
  private
    { Private declarations }
    RCube : TRCube;
    Moves: TMoves;
    No,CurMove : Integer;
    Arrows: array[1..2]of Integer;
    PlayMoves,SetCubeMode : Boolean;
    OpenDialog: TSDLOpenDialog;
    SaveDialog: TSDLSaveDialog;
    procedure ODCallBack(Sender: TSDLOpenDialog; const FileName: string);
    procedure SDCallBack(Sender: TSDLOpenDialog; const FileName: string);
    procedure Init3DDrawing;
    procedure Restore2DDrawing;
  protected
    procedure DoDraw; override;
    procedure KeyUp(var Key: Word; Modifier: TSDLMod); override;
    procedure KeyDown(var Key: Word; Modifier: TSDLMod); override;
  public
    { Public declarations }
    constructor Create(AParent: TSDLObject); override;
    procedure Draw3DScene;
  end;

var
  fmRCube: TfmRCube;

  Mvs: array[-6..6]of string[10]=('Right''','Left''',
           'Back''','Front''','Bottom''','Top''',' ',
           'Top','Bottom','Front','Back','Left','Right');

implementation

uses SysUtils, Classes, Cube3D, Options, SetCubie, gl, glu;

procedure TfmRCube.btLoadClick(Sender: TSDLComponent; AX, AY: Integer);
begin
  OpenDialog.Execute(ODCallBack);
end;

procedure TfmRCube.btMixClick(Sender: TSDLComponent; AX, AY: Integer);
var i,s : Integer;
begin
  for i:= 1 to 20 do
  begin
    s := Random(13)-6;
    if s<>0 then
      RCube.Rotate(s);
  end;
end;

procedure TfmRCube.btNextMoveClick(Sender: TSDLComponent; AX, AY: Integer);
begin
  if (No>0)and(CurMove<=No) then
    if RotateSide(Moves[CurMove]) then
    begin
      Inc(CurMove);
      lbMoves.ItemIndex := CurMove-1;
      if (CurMove>No)and PlayMoves then
      begin
        btPlayClick(Self,0,0);
        btPlay.Down := False;
      end;
    end;
end;

procedure TfmRCube.btOptionsClick(Sender: TSDLComponent; AX, AY: Integer);
begin
  fmOptions.Show;
end;

procedure TfmRCube.btPlayClick(Sender: TSDLComponent; AX, AY: Integer);
begin
  btNextMove.Enabled := PlayMoves;
  btPrevMove.Enabled := PlayMoves;
  PlayMoves := not PlayMoves;
end;

procedure TfmRCube.btPrevMoveClick(Sender: TSDLComponent; AX, AY: Integer);
begin
  if (No>0)and(CurMove>1) then
    if RotateSide(-Moves[CurMove-1]) then
    begin
      Dec(CurMove);
      lbMoves.ItemIndex := CurMove-1;
    end;
end;

procedure TfmRCube.btSaveClick(Sender: TSDLComponent; AX, AY: Integer);
begin
  SaveDialog.Execute(SDCallBack);
end;

procedure TfmRCube.btSetCubeClick(Sender: TSDLComponent; AX, AY: Integer);
begin
  fmSetCubie.RCube := RCube;
  btMix.Enabled := SetCubeMode;
  btSolve.Enabled := SetCubeMode;
  SetCubeMode := not SetCubeMode;
end;

procedure TfmRCube.btSolveClick(Sender: TSDLComponent; AX, AY: Integer);
var i : Integer;
    t : Boolean;
    lc : string;
begin
  t := not btMix.Enabled;
  btMix.Enabled := t;
  btSetCube.Enabled := t;
  btSave.Enabled := t;
  btLoad.Enabled := t;
  btPrevMove.Enabled := not t;
  btPlay.Enabled := not t;
  btNextMove.Enabled := not t;
  lbMoves.Items.Clear;
  No := 0;
  lc := lblMoves.Caption;
  Delete(lc,AnsiPos(':',lc)+1,5);
  if t then Exit;
  No := RCube.Solve(Moves);
  if (No>0)and fmOptions.cbOptimize.Checked then
    No := OptimizeMoves(Moves,No);
  lblMoves.Caption := lc+' '+IntToStr(No);
  CurMove := 1;
  for i := 1 to No do
    lbMoves.Items.Add(Mvs[Moves[i]]);
  lbMoves.ItemIndex := 0;
end;

constructor TfmRCube.Create(AParent: TSDLObject);
begin
  inherited;
  LoadCompFromFile('Main.xfm');
  OpenDialog := TSDLOpenDialog.Create(Gui);
  OpenDialog.Filter := 'Cube files (*.cbe)|All files (*)';
  SaveDialog := TSDLSaveDialog.Create(Gui);
  SaveDialog.DefaultExt := 'cbe';
  fmOptions := TfmOptions.Create(Gui);
  fmSetCubie := TfmSetCubie.Create(Gui);
  SetFocus;
  RCube := TRCube.Create;
  SetActiveCube(RCube);
  Randomize;
  CreateList;
end;

procedure TfmRCube.DoDraw;
begin
  //inherited; Do nothing
end;

procedure TfmRCube.Draw3DScene;
var i,ks: Integer;
begin
  Init3DDrawing;
  if PlayMoves then btNextMoveClick(Self,0,0);
  ks := fmOptions.sbKeyboard.Position;
  for i := 1 to 2 do
    if Arrows[i]<>0 then
      case Arrows[i]of
      SDLK_LEFT: RotateViewFor(-ks,0);
      SDLK_RIGHT: RotateViewFor(ks,0);
      SDLK_UP: RotateViewFor(0,-ks);
      SDLK_DOWN: RotateViewFor(0,ks);
      end;
  DrawCube;
  Restore2DDrawing;
end;

procedure TfmRCube.fmRCubeDestroy(Sender: TSDLComponent);
begin
  RCube.Free;
end;

procedure TfmRCube.fmRCubeMouseMove(Sender: TSDLComponent;
  Modifier: Cardinal; AX, AY: Integer);
begin
  if (KMOD_MBLEFT and Modifier)>0 then
    StartViewRotation(AX,AY);
  RotateView(AX,AY);
end;

procedure TfmRCube.fmRCubeMouseUp(Sender: TSDLComponent; Button: Integer;
  Modifier: Cardinal; AX, AY: Word);
var Pos: TPosition;
begin
  Init3DDrawing;
  Pos := GetClickedPos(AX,AY);
  Restore2DDrawing;
  fmSetCubie.Pos := Pos;
  if not SetCubeMode then
  begin
    if Button=3 then Pos.Side := -Pos.Side;
    if btMix.Enabled then RotateSide(Pos.Side);
  end
  else if (Button=1)and(Pos.Index<>0) then
  begin
    fmSetCubie.Corner := Odd(Pos.Index);
    fmSetCubie.Show;
  end;
  if Button=1 then
    StopViewRotation;
end;

procedure TfmRCube.Init3DDrawing;
begin
  glClear(GL_DEPTH_BUFFER_BIT);
  glViewPort(0,75,670,525);
  glMatrixMode(GL_PROJECTION);
  glPushMatrix;
  glLoadIdentity;
  gluPerspective(45.0, 670/525, 1.0, 100.0);
  glMatrixMode(GL_MODELVIEW);
end;

procedure TfmRCube.KeyDown(var Key: Word; Modifier: TSDLMod);
begin
  if (Key<>SDLK_LEFT)and(Key<>SDLK_RIGHT)and
     (Key<>SDLK_UP)and(Key<>SDLK_DOWN) then inherited
  else if (Arrows[1]<>Key)and(Arrows[2]<>Key) then
    if Arrows[1]=0 then Arrows[1]:= Key
    else Arrows[2] := Key;
end;

procedure TfmRCube.KeyUp(var Key: Word; Modifier: TSDLMod);
begin
  if Key=SDLK_ESCAPE then Gui.SendQuitSignal;
  if (Key<>SDLK_LEFT)and(Key<>SDLK_RIGHT)and
     (Key<>SDLK_UP)and(Key<>SDLK_DOWN) then inherited
  else if (Arrows[1]=Key)or(Arrows[2]=Key) then
    if Arrows[1]=Key then Arrows[1] := 0
    else Arrows[2] := 0;
end;

procedure TfmRCube.lbMovesClick(Sender: TSDLComponent; AX, AY: Integer);
var i : Integer;
begin
  if btMix.Enabled then Exit;
  with lbMoves do
    if (ItemIndex<0)or(ItemIndex=CurMove-1) then Exit;

  if lbMoves.ItemIndex<CurMove then
    for i := CurMove-1 downto lbMoves.ItemIndex+1 do
      RCube.Rotate(-Moves[i])
  else
    for i := CurMove to lbMoves.ItemIndex do
      RCube.Rotate(Moves[i]);
  CurMove := lbMoves.ItemIndex+1;
end;

procedure TfmRCube.ODCallBack(Sender: TSDLOpenDialog; const FileName: string);
var Dat: TFileStream;
    side: integer;
    ID: array[0..2]of Char;
begin
  Dat := TFileStream.Create(FileName,fmOpenRead);
  try
    Dat.Read(ID,3);
    if ID<>'CBE' then raise Exception.Create('Bad file.');
    for side := 1 to 6 do
      Dat.Read(RCube.Side[side].FrontFace,SizeOf(TFaceArray));
  finally
    Dat.Free;
  end;
end;

procedure TfmRCube.Restore2DDrawing;
begin
  glViewPort(0,0,800,600);
  glMatrixMode(GL_PROJECTION);
  glPopMatrix;
  glMatrixMode(GL_MODELVIEW);
end;

procedure TfmRCube.SDCallBack(Sender: TSDLOpenDialog; const FileName: string);
var Dat: TFileStream;
    side: integer;
begin
  Dat := TFileStream.Create(FileName,fmCreate);
  try
    Dat.Write('CBE',3);
    for side := 1 to 6 do
      Dat.Write(RCube.Side[side].FrontFace,SizeOf(TFaceArray));
  finally
    Dat.Free;
  end;
end;

end.
