unit SetCubie;

interface

uses SDLButton, SDLScroll, SDLComboBox, SDLLabel, 
  sdlgui, SDLDraw, SDLForm, sdl, Cube;

type
  TfmSetCubie = class(TSDLForm)
    btCancel: TSDLButton;
    btOK: TSDLButton;
    cbMaster: TSDLComboBox;
    cbSlave: TSDLComboBox;
    SDLLabel1: TSDLLabel;
    SDLLabel2: TSDLLabel;
    SDLScrollBar1: TSDLScrollBar;
    procedure btCancelClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure btOKClick(Sender: TSDLComponent; AX, AY: Integer);
    procedure cbMasterChange(Sender: TSDLComponent);
    procedure cbMasterDrawItem(Sender: TSDLControl; const ItemNo: Integer; Rect: TSDL_Rect; State: Integer);
    procedure cbSlaveDrawItem(Sender: TSDLControl; const ItemNo: Integer; Rect: TSDL_Rect; State: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
    Corner : Boolean;
    Pos : TPosition;
    RCube: TRCube;
    constructor Create(AParent: TSDLObject); override;
    procedure Show;
  end;

var
  fmSetCubie: TfmSetCubie;

implementation

const IndexToCorner: array[0..5,0..3]of Byte =
      ((3,5,7,1),(1,13,15,3),(3,15,17,5),
       (5,17,19,7),(7,19,13,1),(17,15,13,19));
      IndexToEdge: array[0..5,0..3]of Byte =
      ((2,4,6,8),(2,9,14,10),(4,10,16,11),
       (6,11,18,12),(8,12,20,9),(18,16,14,20));
      Cols: array[0..5]of Cardinal=
      ($FFFFFF,$FF00,$FF0000,$80FF,$FF8000,$FFFF00);

procedure TfmSetCubie.btCancelClick(Sender: TSDLComponent; AX,
  AY: Integer);
begin
  Modal := False;
  Visible := False;
  LoseFocus;
end;

procedure TfmSetCubie.btOKClick(Sender: TSDLComponent; AX, AY: Integer);
var c : Byte;
begin
  if Corner then
    c := IndexToCorner[cbMaster.ItemIndex,cbSlave.ItemIndex]
  else
    c := IndexToEdge[cbMaster.ItemIndex,cbSlave.ItemIndex];
  RCube.SetCubie(cbMaster.ItemIndex+1,c,Pos);
end;

procedure TfmSetCubie.cbMasterChange(Sender: TSDLComponent);
begin
  cbSlave.ItemIndex := 0;
end;

procedure TfmSetCubie.cbMasterDrawItem(Sender: TSDLControl;
  const ItemNo: Integer; Rect: TSDL_Rect; State: Integer);
begin
  if (State and stSelected > 0) then
    SDLScreen.FillRect(nil,Rect,$8000);
  InflateRect(Rect,-8,-2);
  SDLScreen.FillRect(nil,Rect,Cols[ItemNo]);
end;

procedure TfmSetCubie.cbSlaveDrawItem(Sender: TSDLControl;
  const ItemNo: Integer; Rect: TSDL_Rect; State: Integer);
var t: Integer;
begin
  if (State and stSelected > 0) then
    SDLScreen.FillRect(nil,Rect,$8000);
  if Corner then
    Rect.w := Rect.w shr 1;
  InflateRect(Rect,-2,-2);
  t := cbMaster.ItemIndex+1;
  SDLScreen.FillRect(nil,Rect,Cols[SideIndex[t,ItemNo]-1]);
  if Corner then
  begin
    Rect.x := Rect.x+Rect.w+4;
    SDLScreen.FillRect(nil,Rect,Cols[SideIndex[t,(ItemNo+1)and 3]-1]);
  end;
end;

constructor TfmSetCubie.Create(AParent: TSDLObject);
begin
  inherited;
  LoadCompFromFile('SetCubie.xfm');
end;

procedure TfmSetCubie.Show;
begin
  cbMaster.ItemIndex := 0;
  cbSlave.ItemIndex := 0;
  Visible := True;
  SetFocus;
end;

end.
